@@ -1,6 +1,6 @@
 ;;; emms-browser.el --- a track browser supporting covers and filtering  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2006-2021  Free Software Foundation, Inc.
+;; Copyright (C) 2006-2023  Free Software Foundation, Inc.
 
 ;; Author: Damien Elmes <emacs@repose.cx>
 ;; Keywords: emms, mp3, mpeg, multimedia
@@ -55,56 +55,6 @@
 
 ;; (require 'emms-browser)
 
-;; Key bindings
-;; -------------------------------------------------------------------
-
-;; C-j             emms-browser-add-tracks-and-play
-;; RET             emms-browser-add-tracks
-;; SPC             emms-browser-toggle-subitems
-;; ^               emms-browser-move-up-level
-;; /               emms-isearch-buffer
-;; 1               emms-browser-collapse-all
-;; 2               emms-browser-expand-to-level-2
-;; 3               emms-browser-expand-to-level-3
-;; 4               emms-browser-expand-to-level-4
-;; <               emms-browser-previous-filter
-;; >               emms-browser-next-filter
-;; ?               describe-mode
-;; C               emms-browser-clear-playlist
-;; E               emms-browser-expand-all
-;; d               emms-browser-view-in-dired
-;; D               emms-browser-delete-files
-;; q               emms-browser-bury-buffer
-;; r               emms-browser-goto-random
-;; n               next-line
-;; p               previous-line
-;; C-/             emms-playlist-mode-undo
-;; <C-return>      emms-browser-add-tracks-and-play
-;; <backtab>       emms-browser-prev-non-track
-;; <tab>           emms-browser-next-non-track
-
-;; s A             emms-browser-search-by-album
-;; s a             emms-browser-search-by-artist
-;; s c             emms-browser-search-by-composer
-;; s s             emms-browser-search-by-names
-;; s t             emms-browser-search-by-title
-;; s p             emms-browser-search-by-performer
-
-;; b 1             emms-browse-by-artist
-;; b 2             emms-browse-by-album
-;; b 3             emms-browse-by-genre
-;; b 4             emms-browse-by-year
-;; b 5             emms-browse-by-composer
-;; b 6             emms-browse-by-performer
-
-;; W a w           emms-browser-lookup-album-on-wikipedia
-
-;; W A w           emms-browser-lookup-artist-on-wikipedia
-
-;; W C w           emms-browser-lookup-composer-on-wikipedia
-
-;; W P w           emms-browser-lookup-performer-on-wikipedia
-
 ;; Displaying covers
 ;; -------------------------------------------------------------------
 
@@ -332,7 +282,12 @@ Can be either a list of small, medium and large images (large
 currently not used), a function which takes a directory and one
 of the symbols `small', `medium' or `large', and should return a
 path to the cover, or nil to turn off cover loading."
-  :type '(choice list function boolean))
+  :type '(choice (list :tag "Cover image names"
+                       (string :tag "Small cover")
+                       (string :tag "Medium cover")
+                       (string :tag "Large cover"))
+                 function
+                 (const :tag "No cover loading")))
 
 (defcustom emms-browser-covers-file-extensions
   '("jpg" "jpeg" "png" "gif" "bmp")
@@ -346,7 +301,13 @@ emms-browser is required."
 
 (defcustom emms-browser-default-covers nil
   "A list of default images to use if a cover isn't found."
-  :type 'list)
+  :type '(choice (const :tag "No defaults" nil)
+                 (list (radio :tag "Small cover" (file :must-match t)
+                              (const :tag "No default" nil))
+                       (radio :tag "Medium cover" (file :must-match t)
+                              (const :tag "No default" nil))
+                       (radio :tag "Large cover" (file :must-match t)
+                              (const :tag "No default" nil)))))
 
 (defcustom emms-browser-comparison-test
   (if (fboundp 'define-hash-table-test)
@@ -414,7 +375,7 @@ Called once for each directory."
 (make-variable-buffer-local 'emms-browser-top-level-hash)
 
 (defvar emms-browser-top-level-type nil
-  "The current mapping type, eg. 'info-artist.")
+  "The current mapping type, eg. \\='info-artist.")
 (make-variable-buffer-local 'emms-browser-top-level-type)
 
 (defvar emms-browser-current-indent nil
@@ -438,7 +399,7 @@ Called once for each directory."
     (define-key map (kbd "RET") #'emms-browser-add-tracks)
     (define-key map (kbd "<C-return>") #'emms-browser-add-tracks-and-play)
     (define-key map (kbd "C-j") #'emms-browser-add-tracks-and-play)
-    (define-key map (kbd "<tab>") #'emms-browser-next-non-track)
+    (define-key map (kbd "<tab>") #'emms-browser-toggle-subitems)
     (define-key map (kbd "<backtab>") #'emms-browser-prev-non-track)
     (define-key map (kbd "d") #'emms-browser-view-in-dired)
     (define-key map (kbd "D") #'emms-browser-delete-files)
@@ -637,10 +598,10 @@ This function can be used as
 
 (defun emms-browser-get-track-field-albumartist (track type)
   "Return TYPE from TRACK with an albumartist-oriented heuristic.
-For 'info-artist TYPE, use 'info-albumartistsort, 'info-albumartist,
-'info-artistsort.
-For 'info-year TYPE, use 'info-originalyear, 'info-originaldate and
-'info-date symbols."
+For \\='info-artist TYPE, use \\='info-albumartistsort,
+\\='info-albumartist, \\='info-artistsort.
+For \\='info-year TYPE, use \\='info-originalyear, \\='info-originaldate and
+\\='info-date symbols."
   (cond ((eq type 'info-artist)
          (or (emms-track-get track 'info-albumartist)
              (emms-track-get track 'info-albumartistsort)
@@ -751,7 +712,8 @@ browser, and hit 'b 1' to refresh.")))
 
 (defun emms-browser-next-mapping-type (current-mapping)
   "Return the next sensible mapping.
-Eg. if CURRENT-MAPPING is currently 'info-artist, return 'info-album."
+Eg. if CURRENT-MAPPING is currently \\='info-artist, return
+ \\='info-album."
   (cond
    ((eq current-mapping 'info-artist) 'info-album)
    ((eq current-mapping 'info-composer) 'info-album)
@@ -770,7 +732,7 @@ Eg. if CURRENT-MAPPING is currently 'info-artist, return 'info-album."
 
 (defun emms-browser-make-bdata-tree-recurse (type level tracks)
   "Build a tree of alists based on a list of tracks, TRACKS.
-For example, if TYPE is 'info-year, return an alist like:
+For example, if TYPE is \\='info-year, return an alist like:
 artist1 -> album1 -> *track* 1.."
   (let* ((next-type (emms-browser-next-mapping-type type))
          (next-level (1+ level))
@@ -828,8 +790,8 @@ return an empty string."
       discnum)))
 
 (defun emms-browser-year-number (track)
-  "Return a string representation of a track's year.
-This will be in the form '(1998) '."
+  "Return a string representation of a track\\='s year.
+This will be in the form \\='(1998) \\='."
   (let ((year (emms-track-get-year track)))
     (if (or (not (stringp year)) (string= year "0"))
         ""
@@ -850,7 +812,7 @@ If no duration is available, return an empty string."
   "Return a browser data item from ALIST.
 DATA should be a list of DB items, or a list of tracks.
 NAME is a name for the DB item.
-TYPE is a category the data is organised by, such as 'info-artist.
+TYPE is a category the data is organised by, such as \\='info-artist.
 LEVEL is the number of the sublevel the db item will be placed in."
   (list (cons 'type type)
         (cons 'level level)
@@ -859,7 +821,7 @@ LEVEL is the number of the sublevel the db item will be placed in."
 
 (defun emms-browser-make-alist (type tracks)
   "Make an alist mapping of TYPE -> TRACKS.
-Items with no metadata for TYPE will be placed in 'misc'"
+Items with no metadata for TYPE will be placed in \\='misc\\='"
   (let (db key existing tracknum)
     (dolist (track tracks)
       (setq key (emms-browser-get-track-field track type))
@@ -881,8 +843,8 @@ Items with no metadata for TYPE will be placed in 'misc'"
 
 (defun emms-browser-make-sorted-alist (type tracks)
   "Return a sorted alist of TRACKS.
-TYPE is the metadata to make the alist by - eg. if it's
-'info-artist, an alist of artists will be made."
+TYPE is the metadata to make the alist by - eg. if it\\='s
+\\='info-artist, an alist of artists will be made."
   (emms-browser-sort-alist
    (emms-browser-make-alist type tracks)
    type))
@@ -985,7 +947,7 @@ Uses `emms-browser-alpha-sort-function'."
 (defun emms-browser-bdata-at-point ()
   "Return the bdata object at point.
 Includes information at point (such as album name), and metadata."
-  (get-text-property (point-at-bol)
+  (get-text-property (line-beginning-position)
                      'emms-browser-bdata))
 
 (defun emms-browser-data-at-point ()
@@ -1112,14 +1074,14 @@ If there is no more subitems to expand, collapse the current node."
 Stops at the next line at the same level, or EOF."
   (when (emms-browser-subitems-visible)
     (let ((current-level (emms-browser-level-at-point))
-          (next-line (point-at-bol 2)))
+          (next-line (line-beginning-position 2)))
       (emms-with-inhibit-read-only-t
        (delete-region next-line
                       (save-excursion
                         (while
                             (emms-browser-find-entry-more-than-level
                              current-level))
-                        (point-at-bol 2)))))))
+                        (line-beginning-position 2)))))))
 
 ;; --------------------------------------------------
 ;; Dealing with the playlist (queuing songs, etc)
@@ -1210,8 +1172,8 @@ After expanding, jump to the currently marked entry."
   (save-excursion
     (emms-browser-find-top-level)
     (emms-with-inhibit-read-only-t
-     (add-text-properties (point-at-bol)
-                          (point-at-eol)
+     (add-text-properties (line-beginning-position)
+                          (line-end-position)
                           (list 'emms-browser-mark t)))))
 
 (defun emms-browser-pop-mark ()
@@ -1223,8 +1185,8 @@ After expanding, jump to the currently marked entry."
         (progn
           (goto-char pos)
           (emms-with-inhibit-read-only-t
-           (remove-text-properties (point-at-bol)
-                                   (point-at-eol)
+           (remove-text-properties (line-beginning-position)
+                                   (line-end-position)
                                    (list 'emms-browser-mark))))
       (message "No mark saved!"))))
 
@@ -1256,7 +1218,7 @@ Return point. If at level one, return the current point."
         (when (emms-browser-subitems-visible)
           (emms-browser-kill-subitems))
         (emms-with-inhibit-read-only-t
-         (goto-char (point-at-bol))
+         (goto-char (line-beginning-position))
          (kill-line 1))
         (unless (eq (emms-browser-bdata-level child-bdata) 1)
           ;; remove the node from the parent, and recurse
@@ -1897,9 +1859,12 @@ the text that it generates."
        ((looking-at "\\([-0-9.]*\\)\\([a-zA-Z]+\\)")
         (let* ((num (match-string 1))
                (spec (match-string 2))
-               (val (cdr (assoc spec specification))))
-          (unless val
+               (val-alist (assoc spec specification))
+               (val (cdr val-alist)))
+          (unless val-alist
             (error "Invalid format character: %s" spec))
+          ;; Value for a valid spec may not exist. Not an error, just nothing to show.
+          (unless val (setq val ""))
           ;; Pad result to desired length.
           (let ((text (format (concat "%" num "s") val)))
             ;; Insert first, to preserve text properties.
@@ -2104,17 +2069,20 @@ If the track is not of TYPE, return t."
   "Directory where to store cover thumbnails.")
 
 (defvar emms-browser-thumbnail-small-size 128
-  "Cover thumbnail will be resized if necessary so that neither width nor height exceed this dimension.")
+  "Cover thumbnail will be resized if necessary so that neither
+  width nor height exceed this dimension.")
 (defvar emms-browser-thumbnail-medium-size 256
-  "Cover thumbnail will be resized if necessary so that neither width nor height exceed this dimension.")
+  "Cover thumbnail will be resized if necessary so that neither
+  width nor height exceed this dimension.")
 (defvar emms-browser-thumbnail-large-size 1024 ; Emms does not use large covers as of 2017-11-26.
-  "Cover thumbnail will be resized if necessary so that neither width nor height exceed this dimension.")
+  "Cover thumbnail will be resized if necessary so that neither
+  width nor height exceed this dimension.")
 
 (defun emms-browser-thumbnail-filter-default (dir)
-  "Select covers containing 'front' or 'cover' in DIR.
-If none was found, fallback on `emms-browser-thumbnail-filter-all'.
+  "Select covers containing \\='front\\=' or \\='cover\\=' in DIR.
+If none was found, fallback on `emms-browser-thumbnail-filter-all\\='.
 
-See `emms-browser-thumbnail-filter'."
+See `emms-browser-thumbnail-filter\\='."
   (when (file-directory-p dir)
     (let ((ls (directory-files dir t nil t))
           (case-fold-search t)
@@ -2126,7 +2094,7 @@ See `emms-browser-thumbnail-filter'."
       covers)))
 
 (defun emms-browser-thumbnail-filter-all (dir)
-  "Return the list of all files with `emms-browser-covers-file-extensions' in DIR.
+  "Return the list of all files with `emms-browser-covers-file-extensions\\=' in DIR.
 
 See `emms-browser-thumbnail-filter'."
   (let (covers)
@@ -2134,7 +2102,9 @@ See `emms-browser-thumbnail-filter'."
       (setq covers (append (file-expand-wildcards (expand-file-name (concat "*." ext) dir)) covers)))))
 
 (defvar emms-browser-thumbnail-filter 'emms-browser-thumbnail-filter-default
-  "This filter must hold a function that takes a directory argument and returns a list of cover file names.
+  "This filter must hold a function that takes a directory argument and returns
+a list of cover file names.
+
 The list will be processed by `emms-browser-cache-thumbnail'.
 See also `emms-browser-thumbnail-filter-default'.")
 
@@ -2144,19 +2114,19 @@ See also `emms-browser-thumbnail-filter-default'.")
 (defun emms-browser-cache-thumbnail (dir size)
   "Return cached cover SIZE for album in DIR.
 
-SIZE must be 'small, 'medium or 'large.  It will determine the
+SIZE must be \\='small, \\='medium or \\='large.  It will determine the
 resolution of the cached file.  See the variables
-`emms-browser-thumbnail-SIZE-size'.
+`emms-browser-thumbnail-SIZE-size\\='.
 
 If cover is not cached or if cache is out-of-date, re-cache it.
 If both the width and the height of the cover are smaller than
-`emms-browser-thumbnail-SIZE-size', it need not be cached and
+`emms-browser-thumbnail-SIZE-size\\=', it need not be cached and
 will be used directly.
 
 Emms assumes that you have one album per folder. This function
 will always use the same cover per folder.
 
-`emms-browser-covers' can be `fset' to this function."
+`emms-browser-covers\\=' can be `fset\\=' to this function."
   (if (eq size 'large)
       ;; 'large is unused for now. Return empty.
       nil
